%   Exporter Heterogeneity and Price Discrimination: A Quantitative View
%   Jae Wook Jung, Ina Simonovska, and Ariel Weinberger
%
%   code for generating tables
%   1. domestic sales advantage of exporters over non-exporters
%   2. mark-up (value-added) advantage of exporters over non-exporters
%   3. (over-identification only) percentile differences of two advantages: 99th-90th, 90th-10th, 90th-50th 

function [] = gen_plots_firmmmt(sigma,theta,L,y,tech,taustheta,tradeshare,N,seed)

% restore parameters
Ncountry = N.country;
Ndraws = N.draws;
Nmmts = N.mmts;

% restore trade costs
taus = taustheta.^(1./theta);

% Psi_j = relative cbar_j to cbar of the reference country
Psi = zeros(Ncountry,1);
for j = 1:Ncountry
    Psi(j) = ((y(j)./y(end)).*(sum (tech.*taus(:,end).^(-theta),1)./sum (tech.*taus(:,j).^(-theta),1))).^(1/(theta+1)); % eq(29)
end

% recover cbar_j
cbar = zeros(Ncountry,1);
cbar(Ncountry) = 1; % cbar of reference country is assumed as one WOLG. note that only relative costs to cbars are used in the simulation.
cbar = Psi.*cbar(Ncountry);

% price and cost series for each firm in the reference country
price = zeros(Ncountry,Ndraws);
cost = zeros(Ncountry,Ndraws);
Idx = zeros(Ncountry,Ndraws);
Idxnum = zeros(Ncountry,1);
tij = zeros(Ncountry,Ndraws);

% generate random draw for firms
stream = RandStream.create('mt19937ar','Seed',seed);
reset(stream);
RandStream.setGlobalStream(stream);
randomdraw = sort(rand(Ndraws,1));

% generate cost for each firm
cost = (randomdraw.^(1/theta)*(cbar(Ncountry).*taus(Ncountry,:)))';

% index for active firms
Idx(cost <= cbar*ones(1,Ndraws)) = 1;
Idx(tradeshare(Ncountry,:)==0,:) = 0;

% number of active firms in each destination m
for m = 1:Ncountry
    Idxnum(m) = size(find(Idx(m,:)>0),2);
end

if sigma == 1 % sigma = 1 case is Simonovska(2015) model
    price = Idx.*(cbar*ones(1,Ndraws).*cost).^(1/2);
else
    for m = 1:Ncountry % destination
        if Idxnum(m) > 0 % for only active firms
            price(m,Idxnum(m)) = Brentzero(eps,cbar(m)-eps,eps,eps,@(p) (1-sigma).*p + sigma.*cost(m,Idxnum(m)) - p.^(sigma+1).*cbar(m).^(-sigma)); % eq(7)
        end
        for n = 1:Idxnum(m)-1 % ascending index of active firms in destination m
            nn = Idxnum(m)-n; % descending index of active firms in destination m
            if Idx(m,nn) == 1 % for only active firms
                % 3rd-order approximation of eq(7)
                price(m,nn) = price(m,nn+1) + (cost(m,nn+1) - cost(m,nn))*(sigma./((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma))) ... %1st order
                    + ((cost(m,nn+1) - cost(m,nn)).^2./2).*(sigma.^3.*(sigma+1)*price(m,nn+1).^(sigma-1)* cbar(m).^(-sigma))./(((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma)).^3) ... %2nd order
                    + ((cost(m,nn+1) - cost(m,nn)).^3./6).*( (sigma.^3.*(sigma+1).*(sigma-1).*price(m,nn+1).^(sigma-2)* cbar(m).^(-sigma))./(((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma)).^3) ... %3rd order first
                    - (sigma.^3.*(sigma+1).*price(m,nn+1).^(sigma-1)* cbar(m).^(-sigma).*3.*(((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma)).^2).*((-(sigma+1).*sigma.*price(m,nn+1).^(sigma-1).*cbar(m).^(-sigma))) ... %3rd order second numerator
                    ./(((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma)).^6))) ... % 3rd order second denominator
                    .*(sigma./((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma))); % 3rd order 1st der.
            end
        end
    end
end

% new firm level variable = price/cbar
tij = price./(cbar*ones(1,Ndraws));
%vij = cost./(cbar*ones(1,Ndraws));

% computing productivity (value-added, in logs) adavantage of exporters over non-exporters, eq(30)

t_1 = tij.^(1-sigma);
t_2 = tij.^(-sigma);
t_1(isinf(t_1)) = 0;
t_2(isinf(t_2)) = 0;

tii = tij(Ncountry,:);
tii_1 = t_1(Ncountry,:);
tii_2 = t_2(Ncountry,:);

% sales of each firm across destinations
salesm = Idx.*((cbar.*L)*ones(1,Ndraws)).*(t_1-tij);
totsales = sum(salesm,1);

% domestic sales of firms in the reference country
domsales = salesm(end,:);

Mavgsales = (theta./((sigma).^theta)).*...
        trapz(tii(1:Ndraws),((tii_1(1:Ndraws) - tii(1:Ndraws)))...
    .*((tii(1:Ndraws).^(sigma+1) + (sigma-1).*tii(1:Ndraws)).^(theta-1).*((sigma+1).*tii(1:Ndraws).^(sigma) + (sigma-1))),2);

% mark-up advantage of advantage of exporters over non-exporters
mu = sigma./(tii.^sigma-(1-sigma)); % mark-up
mu(isinf(mu)) = 0;

% average markup
Mavgmkup = 1+(theta./((sigma).^theta)).*...
         trapz(tii(1:Ndraws),(mu(1:Ndraws)-1)...
     .*((tii(1:Ndraws).^(sigma+1) + (sigma-1).*tii(1:Ndraws)).^(theta-1).*((sigma+1).*tii(1:Ndraws).^(sigma) + (sigma-1))),2);        
        
relsales = log(domsales./Mavgsales);
relmkup = log(mu./Mavgmkup);

figure()
[n, xout]=hist(relsales, 200);
bar(xout,n/sum(n))
title('Chilean Domestic Sales(relative to the mean, in logs) Distribution (Simulation)', 'FontSize', 12)
% xlabel('Sales')
xlim([-12 4])
print('-dpng', 'logsales_Chile'); 

figure()
[n, xout]=hist(relmkup, 200);
bar(xout,n/sum(n))
title('Chilean Markups(relative to the mean, in logs) Distribution (Simulation)', 'FontSize', 12)
% xlabel('Sales')
xlim([min(relmkup) max(relmkup)])
print('-dpng', 'logmkups_Chile'); 

end