%   Exporter Heterogeneity and Price Discrimination: A Quantitative View
%   Jae Wook Jung, Ina Simonovska, and Ariel Weinberger
%
%   code for generating tables
%   1. domestic sales advantage of exporters over non-exporters
%   2. mark-up (value-added) advantage of exporters over non-exporters
%   3. (over-identification only) percentile differences of two advantages: 99th-90th, 90th-10th, 90th-50th 

function [mmts,pred,expintmat] = gen_tables_firmmmt(sigma,theta,L,y,tech,taustheta,tradeshare,N,seed)

% restore parameters
Ncountry = N.country;
Ndraws = N.draws;
Nmmts = N.mmts;

% restore trade costs
taus = taustheta.^(1./theta);

% Psi_j = relative cbar_j to cbar of the reference country
Psi = zeros(Ncountry,1);
for j = 1:Ncountry
    Psi(j) = ((y(j)./y(end)).*(sum (tech.*taus(:,end).^(-theta),1)./sum (tech.*taus(:,j).^(-theta),1))).^(1/(theta+1)); % eq(29)
end

% recover cbar_j
cbar = zeros(Ncountry,1);
cbar(Ncountry) = 1; % cbar of reference country is assumed as one WOLG. note that only relative costs to cbars are used in the simulation.
cbar = Psi.*cbar(Ncountry);

% price and cost series for each firm in the reference country
price = zeros(Ncountry,Ndraws);
cost = zeros(Ncountry,Ndraws);
Idx = zeros(Ncountry,Ndraws);
Idxnum = zeros(Ncountry,1);
tij = zeros(Ncountry,Ndraws);

% generate random draw for firms
stream = RandStream.create('mt19937ar','Seed',seed);
reset(stream);
RandStream.setGlobalStream(stream);
randomdraw = sort(rand(Ndraws,1));

% generate cost for each firm
cost = (randomdraw.^(1/theta)*(cbar(Ncountry).*taus(Ncountry,:)))';

% index for active firms
Idx(cost <= cbar*ones(1,Ndraws)) = 1;
Idx(tradeshare(Ncountry,:)==0,:) = 0;

% number of active firms in each destination m
for m = 1:Ncountry
    Idxnum(m) = size(find(Idx(m,:)>0),2);
end

if sigma == 1 % sigma = 1 case is Simonovska(2015) model
    price = Idx.*(cbar*ones(1,Ndraws).*cost).^(1/2);
else
    for m = 1:Ncountry % destination
        if Idxnum(m) > 0 % for only active firms
            price(m,Idxnum(m)) = Brentzero(eps,cbar(m)-eps,eps,eps,@(p) (1-sigma).*p + sigma.*cost(m,Idxnum(m)) - p.^(sigma+1).*cbar(m).^(-sigma)); % eq(7)
        end
        for n = 1:Idxnum(m)-1 % ascending index of active firms in destination m
            nn = Idxnum(m)-n; % descending index of active firms in destination m
            if Idx(m,nn) == 1 % for only active firms
                % 3rd-order approximation of eq(7)
                price(m,nn) = price(m,nn+1) + (cost(m,nn+1) - cost(m,nn))*(sigma./((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma))) ... %1st order
                    + ((cost(m,nn+1) - cost(m,nn)).^2./2).*(sigma.^3.*(sigma+1)*price(m,nn+1).^(sigma-1)* cbar(m).^(-sigma))./(((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma)).^3) ... %2nd order
                    + ((cost(m,nn+1) - cost(m,nn)).^3./6).*( (sigma.^3.*(sigma+1).*(sigma-1).*price(m,nn+1).^(sigma-2)* cbar(m).^(-sigma))./(((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma)).^3) ... %3rd order first
                    - (sigma.^3.*(sigma+1).*price(m,nn+1).^(sigma-1)* cbar(m).^(-sigma).*3.*(((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma)).^2).*((-(sigma+1).*sigma.*price(m,nn+1).^(sigma-1).*cbar(m).^(-sigma))) ... %3rd order second numerator
                    ./(((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma)).^6))) ... % 3rd order second denominator
                    .*(sigma./((1-sigma) - (sigma + 1)*price(m,nn+1).^sigma * cbar(m).^(-sigma))); % 3rd order 1st der.
            end
        end
    end
end

% new firm level variable = price/cbar
tij = price./(cbar*ones(1,Ndraws));
%vij = cost./(cbar*ones(1,Ndraws));

% make exporter index for exporters of the reference country
cbartaux = (ones(Ncountry,1)*cbar')./taus; % cost thresholds for marginal exporters to each destination
[cbartauxUS,Idxe] = max(cbartaux(Ncountry,1:Ncountry-1));  % cbartauxUS(=ctilde_x in the paper): cost thresholds for marginal exporter exporting to at least one destination
Idxjvx = size(find(Idx(Idxe,:,end)==1),2); % index for the marginal exporter in the reference country

Xi = cbartauxUS./(cbar(Ncountry)); % relative cost of the marginal exporter to the marginal domestic producer
if Xi > 1
    Xi = 1; % Xi cannot greater than one
    cbartauxUS = cbar(Ncountry);
end

% relative measure of exporters in the reference country
Mexp = (cbartauxUS./(cbar(Ncountry))).^theta; % eq(35)

cbartauxUSmat = zeros(Ncountry,1); % ctilde_x matrix for all source countries
Idxemat = zeros(Ncountry,1);
for i = 1:Ncountry
    cbartauxtemp = cbartaux(i,:);
    if i == 1
    cbartaux2 = cbartauxtemp(i+1:end);
    elseif i == Ncountry
        cbartaux2 = cbartauxtemp(1:i-1);
    else
        cbartauxtemp1 = cbartauxtemp(1:i-1);
        cbartauxtemp2 = cbartauxtemp(i+1:end);
        cbartaux2 = [cbartauxtemp1';cbartauxtemp2']';
    end
    [cbartauxUSmat(i),Idxemat(i)] = max(cbartaux2); % easiest destination for exporters
    if Idxemat(i) >= i
        Idxemat(i) = Idxemat(i) + 1;
    end
    %[cbartauxall,Idxe2] = min(cbartaux2(Ncountry,1:Ncountry-1)); % toughest destination for exporters
end

Ximat = cbartauxUSmat./cbar;
Mexpmat = Ximat.^theta;
% disp('Mexp for Chile:')
% Mexpmat(Ncountry)
% disp('Mexp for US:')
% Mexpmat(61)
% disp('avg. Mexp:')
% mean(Mexpmat)
% disp('GDP weighted avg. Mexp:')
% Y = y.*L;
% yweight = Y./sum(Y);
% mean(Mexpmat.*yweight)


% computing productivity (value-added, in logs) adavantage of exporters over non-exporters, eq(30)

t_1 = tij.^(1-sigma);
t_2 = tij.^(-sigma);
t_1(isinf(t_1)) = 0;
t_2(isinf(t_2)) = 0;

tii = tij(Ncountry,:);
tii_1 = t_1(Ncountry,:);
tii_2 = t_2(Ncountry,:);

% Mprod0: average exporter logged productivity
Mprod0 = trapz(tii(1:Idxjvx),theta./(sigma.^(theta).*Xi.^(theta)) ...
.*(...
log(sum(Idx(:,1:Idxjvx).*((Psi.*L)*ones(1,Idxjvx)).*(t_1(:,1:Idxjvx) - tij(:,1:Idxjvx)),1))...
-log(sum(Idx(:,1:Idxjvx).*((Psi.*L)*ones(1,Idxjvx)).*(t_2(:,1:Idxjvx) - 1).*((tij(:,1:Idxjvx).^(sigma+1) + (sigma-1).*tij(:,1:Idxjvx))./(sigma.*y(Ncountry))),1))...
)...
.*((tii(:,1:Idxjvx).^(sigma+1) + (sigma-1).*tii(:,1:Idxjvx)).^(theta-1).*((sigma+1).*tii(:,1:Idxjvx).^(sigma) + (sigma-1))),2);

% Mprod1: average non-exporter logged productivity

if sigma == 1
    Mprod1 = log(y(Ncountry)) + 1./(2*theta) + log(tii(Idxjvx)).* (Xi^(theta)./(1-Xi.^(theta)));
else
Mprod1 = trapz(tii(Idxjvx+1:Ndraws),theta./(sigma.^(theta).*(1-(Xi).^(theta))) ...
.*Idx(Ncountry,1).*(log(tii_1(Idxjvx+1:Ndraws) - tii(Idxjvx+1:Ndraws)) - log(tii_2(Idxjvx+1:Ndraws)-1) - log((tii(Idxjvx+1:Ndraws).^(sigma+1) + (sigma-1).*tii(Idxjvx+1:Ndraws))./sigma) + log(y(Ncountry))) ...
.*((tii(Idxjvx+1:Ndraws).^(sigma+1) + (sigma-1).*tii(Idxjvx+1:Ndraws)).^(theta-1).*((sigma+1).*tii(Idxjvx+1:Ndraws).^(sigma) + (sigma-1))),2);
end

Mprod = Mprod0 - Mprod1;
clear Mprod0 Mprod1

% computing domestic sales advantage of exporters over non-exporters, eq(31)
% Msales0: average non-exporter domestic sales
Msales0 = (1-Xi.^(theta)).*...
    trapz(tii(1:Idxjvx),((tii_1(1:Idxjvx) - tii(1:Idxjvx)))...
    .*((tii(1:Idxjvx).^(sigma+1) + (sigma-1).*tii(1:Idxjvx)).^(theta-1).*((sigma+1).*tii(1:Idxjvx).^(sigma) + (sigma-1))),2);
% Msales1: average exporter domestic sales
Msales1 = Xi.^(theta).*...
    trapz(tii(Idxjvx+1:Ndraws),((tii_1(Idxjvx+1:Ndraws) - tii(Idxjvx+1:Ndraws)))...
    .*((tii(Idxjvx+1:Ndraws).^(sigma+1) + (sigma-1).*tii(Idxjvx+1:Ndraws)).^(theta-1).*((sigma+1).*tii(Idxjvx+1:Ndraws).^(sigma) + (sigma-1))),2);
Msales = Msales0 ./ Msales1;

% sales of each firm across destinations
salesm = Idx.*((cbar.*L)*ones(1,Ndraws)).*(t_1-tij);
totsales = sum(salesm,1);

% domestic sales of firms in the reference country
domsales = salesm(end,:);

% standard deviation of domestic sales
sdlogsales = std(log(domsales));

% mark-up advantage of advantage of exporters over non-exporters
mu = sigma./(tii.^sigma-(1-sigma)); % mark-up
mu(isinf(mu)) = 0;

% average markup
Mavgmkup = (theta./((sigma).^theta)).*...
         trapz(tii(1:Ndraws),(mu(1:Ndraws)-1)...
     .*((tii(1:Ndraws).^(sigma+1) + (sigma-1).*tii(1:Ndraws)).^(theta-1).*((sigma+1).*tii(1:Ndraws).^(sigma) + (sigma-1))),2);

% Mavgmkup0: average non-exporter mark-up
Mavgmkup0 = (theta./((sigma.*Xi).^theta)).*...
    trapz(tii(1:Idxjvx),log((mu(1:Idxjvx)))...
    .*((tii(1:Idxjvx).^(sigma+1) + (sigma-1).*tii(1:Idxjvx)).^(theta-1).*((sigma+1).*tii(1:Idxjvx).^(sigma) + (sigma-1))),2);
% Mavgmkup1: average exporter mark-up
Mavgmkup1 = (theta./(sigma.^theta.*(1-Xi.^(theta)))).*...
    trapz(tii(Idxjvx+1:Ndraws),log((mu(Idxjvx+1:Ndraws)))...
    .*((tii(Idxjvx+1:Ndraws).^(sigma+1) + (sigma-1).*tii(Idxjvx+1:Ndraws)).^(theta-1).*((sigma+1).*tii(Idxjvx+1:Ndraws).^(sigma) + (sigma-1))),2);

% computing percentile differences of two advantages

% log of total markup
logtotmkup = log(sum(Idx.*((Psi.*L)*ones(1,Ndraws)).*(t_1 - tij),1))...
-log(sum(Idx.*((Psi.*L)*ones(1,Ndraws)).*(t_2 - 1).*((tij.^(sigma+1) + (sigma-1).*tij)./(sigma)),1));

% 90% - 10% difference
% sales
Msales9010 = domsales(floor((1-0.90)*Ndraws)) ./ domsales(floor((1-0.10)*Ndraws));%logtotsales(floor((1-0.90)*Ndraws)) - logtotsales(floor((1-0.10)*Ndraws));
% markup
Mmkup9010 = logtotmkup(floor((1-0.90)*Ndraws)) - logtotmkup(floor((1-0.10)*Ndraws));

% 90% - 50% difference
% sales
Msales9050 = domsales(floor((1-0.90)*Ndraws)) ./ domsales(floor((1-0.50)*Ndraws));
% markup
Mmkup9050 = logtotmkup(floor((1-0.90)*Ndraws)) - logtotmkup(floor((1-0.50)*Ndraws));

% 99% - 90% difference
% sales
Msales9990 = domsales(floor((1-0.99)*Ndraws)) ./ domsales(floor((1-0.90)*Ndraws));
% markup
Mmkup9990 = logtotmkup(floor((1-0.99)*Ndraws)) - logtotmkup(floor((1-0.90)*Ndraws));

%%% pass-through elasticity, eq(24)
rho0 = (theta./((sigma).^theta)).*trapz(tii,...
(sigma .*(tii.^sigma)./ ((sigma-1) + (sigma+1) .* tii.^sigma)).*(tii_1-tii).*((tii.^(sigma+1) + (sigma-1).*tii).^(theta-1).*((sigma+1).*tii.^(sigma) + (sigma-1))),2);

rho1 = (theta./((sigma).^theta)).*trapz(tii(1:Ndraws),...
(tii_1(1:Ndraws)-tii(1:Ndraws)).*((tii(1:Ndraws).^(sigma+1) + (sigma-1).*tii(1:Ndraws)).^(theta-1).*((sigma+1).*tii(1:Ndraws).^(sigma) + (sigma-1))),2);
rho = rho0./rho1;

%%% average price-income elasticity
lambdaijreal = tradeshare./(ones(N.country,1)*sum(tradeshare,1));
lambdajj = diag(lambdaijreal);
lambdajjref = lambdajj(end);

Mepw_simple = ((theta./(sigma.^theta))).* ...
    ((1- (1+theta.*lambdajjref)./(theta+1)).*trapz(tii(1:Ndraws),((sigma./((1-sigma).*(tii(1:Ndraws)).^(-sigma)-(sigma+1))).*((tii(1:Ndraws)).^(sigma+1)+(sigma-1).*(tii(1:Ndraws))).^(theta-1).*...
    ((sigma+1).*((tii(1:Ndraws)).^sigma)+(sigma-1))),2) +  ...
    trapz(tii(1:Ndraws),(((tii(1:Ndraws)).^(sigma+1)+(sigma-1).*(tii(1:Ndraws))).^(theta-1).*((sigma+1).*((tii(1:Ndraws)).^sigma)+(sigma-1))),2));


%%% sales weighted average income-price elasticity
Mepw_sw = rho.*(1 - (1+theta.*lambdajjref)./(1+theta))+1;

%%% export intensity
expsales = totsales - domsales;
expint = expsales./totsales;
 
kind = zeros(9,1);
expintrank = zeros(9,1);
for k = 1:9
    kind(k) = size(find(expint(:)<(k.*0.1)),1);
    expintrank(k) = (cost(Ncountry,kind(k))./cbar(Ncountry)).^theta;    
end
%disp('expot intensity from 0-10 perc to 80-90 perc')
expintmat = zeros(7,1);
expintmat(1) = expintrank(1);
for k2 = 1:5
   expintmat(k2+1) = expintrank(k2+1) - expintrank(k2);
end
expintmat(7) = 1 - sum(expintmat);
expintmat = expintmat.*100;

% moments
if Nmmts == 2 % exactly identified case
    mmts = [Msales; Mprod]; 
elseif Nmmts == 8 % over-identified case
    mmts= [Msales; Mprod; Msales9010; Msales9050; Msales9990; Mmkup9010; Mmkup9050; Mmkup9990];
end

vv = isinf(mmts); % ignore infinite moment case
mmts(vv) = NaN;

% prediction variables
pred = [rho;Mepw_simple;Mexpmat(Ncountry)*100;Mexpmat(61)*100;mean(Mexpmat)*100;Mavgmkup;sdlogsales];