%   Exporter Heterogeneity and Price Discrimination: A Quantitative View
%   Jae Wook Jung, Ina Simonovska, and Ariel Weinberger
%
%   code of generating price difference moments in subroutine
%   1. mean dni
%   2. mean dni_85th
%   3. cov(dni, log(wn/wi))


function [mmts] = gen_mmts_pricemmt(sigma,theta,y,tech,taustheta,tradeshare,N,seed,flag)

tic;

% restore parameters
commoncutoff = flag.commoncutoff;
second = flag.second;
Ncountry = N.country;
Ndraws = N.draws;
Nmmts = N.mmts;
Ngood = N.good;
Nsample = N.sample;

% restore trade costs
taus = taustheta.^(1./theta);

% Psi_j = relative cbar_j to cbar of the reference country
Psi = zeros(Ncountry,1);
for j = 1:Ncountry
    Psi(j) = ((y(j)./y(end)).*(sum (tech.*taus(:,end).^(-theta),1)./sum (tech.*taus(:,j).^(-theta),1))).^(1/(theta+1)); % eq(29)
end

% recover cbar_j
cbar = zeros(Ncountry,1);
cbar(Ncountry) = 1; % cbar of reference country is assumed as one WOLG. note that only relative costs to cbars are used in the simulation.
cbar = Psi.*cbar(Ncountry);

% find number of sources that export certain number of goods commonly available in destinations
cbartaux = (ones(Ncountry,1)*cbar')./taus; % cbar/taus
[cbartauxsort,~] = sort(cbartaux,2,'descend');
cbarnleast = min(cbartauxsort(:,commoncutoff));

% common price and cost series for every country (destination,firm,source)
common = zeros(1,Ncountry);
Idxnum = zeros(Ncountry,Ncountry);
Idxnumcommon = zeros(Ncountry,Ncountry);
Idxcommon = zeros(Ncountry,1);
Idxdest = zeros(Ncountry,1);

% generate random draw for firms
stream = RandStream.create('mt19937ar','Seed',seed);
reset(stream);
RandStream.setGlobalStream(stream);
randomdraw = sort(rand(Ndraws,1));

cbarn = (cbarnleast-eps)./(randomdraw(1).^(1/theta)); % destination specific cbar

% cost draws for the domestic market, relative to the highest cbar in the world
costjj = (randomdraw.^(1/theta)*cbarn)';
clear randomdraw


for l = 1:Ncountry % source
    
    Idxlnuml = size(find(costjj <=cbar(l)),2); % the number of domestic producers in l
    
    % restrict the range of solving price for the number of domestic survivers
    costl = ((costjj(1:Idxlnuml))'*taus(l,:))';
    Idxl = zeros(Ncountry,Idxlnuml);
    Idxl(costl <= cbar*ones(1,Idxlnuml)) = 1; % number of active firms from each source l
    Idxlnum = zeros(Ncountry,1);
    for m = 1:Ncountry
        Idxlnum(m) = size(find(Idxl(m,:)>0),2); % number of destinations firms operating in
    end
    
    % consider only if the number of accessible market is greater than cutoff
    Idxldest = size(find(Idxlnum(:)>0),1);
    Idxlnumcommon = zeros(Ncountry,1);
    if Idxldest >= commoncutoff
        
        % consider goods available at home and at least in the U.S
        % restrict price simulation for firms that can access to common
        % markets
        n = 1;
        Idxlnumtemp = Idxldest;
        while Idxlnumtemp >= commoncutoff
            n = n + 1;
            if n > Idxlnuml
                break;
            end
            Idxlnumtemp2 = zeros(Ncountry,1);
            for m = 1:Ncountry
                Idxlnumtemp2(m) = size(find(Idxl(m,n)>0),2);
            end
            Idxlnumtemp = size(find(Idxlnumtemp2(:)>0),1);
            
        end
        Idxlnummin = n-1;
        
        for m = 1:Ncountry
            Idxlnumcommon(m) = size(find(Idxl(m,1:Idxlnummin)>0),2);
        end
        
        % number of firms that can access to common markets
        Idxcommon(l) = Idxlnummin;
        if Idxlnummin > 0
            pricel = zeros(Ncountry,Idxlnummin);
            for m = 1:Ncountry % destination
                Idxlnummintemp = min(Idxlnum(m),Idxlnummin);
                
                if Idxlnum(m) > 0
                    pricel(m,Idxlnummintemp) = fzero(@(p) (1-sigma).*p + sigma.*costl(m,Idxlnummintemp) - p.^(sigma+1).*cbar(m).^(-sigma),[eps cbar(m)-eps],optimset('TolX',eps,'Display','off'));
                    
                    for n = 1:Idxlnummintemp - 1 % ascending index of active firms in destination m
                        nn = Idxlnummintemp - n; % descending index of active firms in destination m
                        % 3rd-order approximation of eq(7)
                        pricel(m,nn) = pricel(m,nn+1) + (costl(m,nn+1) - costl(m,nn))*(sigma./((1-sigma) - (sigma + 1)*pricel(m,nn+1).^sigma * cbar(m).^(-sigma))) ... %1st order
                            + ((costl(m,nn+1) - costl(m,nn)).^2./2).*(sigma.^3.*(sigma+1)*pricel(m,nn+1).^(sigma-1)* cbar(m).^(-sigma))./(((1-sigma) - (sigma + 1)*pricel(m,nn+1).^sigma * cbar(m).^(-sigma)).^3) ... %2nd order
                            + ((costl(m,nn+1) - costl(m,nn)).^3./6).*( (sigma.^3.*(sigma+1).*(sigma-1).*pricel(m,nn+1).^(sigma-2)* cbar(m).^(-sigma))./(((1-sigma) - (sigma + 1)*pricel(m,nn+1).^sigma * cbar(m).^(-sigma)).^3) ... %3rd order first
                            - (sigma.^3.*(sigma+1).*pricel(m,nn+1).^(sigma-1)* cbar(m).^(-sigma).*3.*(((1-sigma) - (sigma + 1)*pricel(m,nn+1).^sigma * cbar(m).^(-sigma)).^2).*((-(sigma+1).*sigma.*pricel(m,nn+1).^(sigma-1).*cbar(m).^(-sigma))) ... %3rd order second numerator
                            ./(((1-sigma) - (sigma + 1)*pricel(m,nn+1).^sigma * cbar(m).^(-sigma)).^6))) ... % 3rd order second denominator
                            .*(sigma./((1-sigma) - (sigma + 1)*pricel(m,nn+1).^sigma * cbar(m).^(-sigma))); % 3rd order 1st der.
                    end
                end
                
            end
        end
        
        common = [common;pricel'];
        
    end
    Idxnum(l,:) = Idxlnum';
    Idxdest(l) = Idxldest;
    Idxnumcommon(l,:) = Idxlnumcommon';
end
common = common(2:end,:); % stack commonly available good prices
clear l Idxlnum Idxl costl pricel tijl vijl Idxlnummin

% construct the set of commonly available goods 

% common = ones(1,Ncountry);
% Idxnuml = zeros(Ncountry,1);
% for l = 1:Ncountry % source
%     Idxnuml(l) =  Idxnum(Idxmin(l),l);%size(find(cost(Idxmin(l),:,l) <= cbarmin(l)),2);
%     common = [common;price(:,1:Idxnuml(l),l)'];
% end
% Idxnuml
% common = common(2:end,:);

% ignore goods not available in the reference country
numcommon = size(common,1);
usnonzero = find(common(:,Ncountry)~=0);
common = common(usnonzero,:);
numcommonUS = size(common,1);

%
common_EIU = common;

% %%% calculate the relative prices to the U.S.
% rel_price_m = zeros(Ncountry,110);
% rel_price_m = (common_EIU./((common_EIU(:,Ncountry))*ones(1,Ncountry)))';
%%%
%clear i
% X = repmat([y L],length(gooddummy),1);
% CD = repmat(countrydummy,length(gooddummy),1);
% G = repmat(gooddummy',length(countrydummy),1);
% G = reshape(G,length(X),1);
% P = reshape(rel_price_m,length(G),1);
% 
% whole = [P X CD G];
% clear P X CD G rel_price_m countrydummy gooddummy common_EIU
%%%%%
% randomly sample 110 goods
sample = sampling_price(common_EIU,Ncountry,Ngood,seed,Nsample,Idxcommon);

if length(sample) >= Ngood * Ncountry

    flag.rs = 1;
    mmts = zeros(Nmmts,Nsample);    
    rel_price = reshape(sample(:,1),[Ncountry,Ngood,Nsample]);
    for s = 1:Nsample
        mmts(:,s) = gen_mmts_pricemmt2(rel_price(:,:,s),y,tradeshare,second,Nmmts);
    end    

else
    disp('one price sample')    
    flag.rs = 0;
    rel_price = reshape(sample,Ncountry,length(sample)./Ncountry);
    mmts = gen_mmt_pricemmt2(rel_price,y,tradeshare,second,Nmmts);
    mmts = mmts*ones(1,Nsample);
end

temp = toc;
disp(['Time to finish one outer loop:' num2str(temp) 'seconds' ])

end