%   Exporter Heterogeneity and Price Discrimination: A Quantitative View
%   Jae Wook Jung, Ina Simonovska, and Ariel Weinberger
%
%   main driver for price difference estimation
%
%   This code is written for Matlab R2015a or later.
%   lines using Parallel computing toolbox are not compatible with earlier
%   versions of Matlab than R2015a.

clc
clear all;
close all;

% set the number of moments in the simulation
N.mmts = 3;
% Over-Identification with Firm Moments: 3 moments
% Exact-Identification with Firm Moments: 1 moments

% parameters and flags
 
%%% algorithm
flag.approx = 1; %approximation for price equation 0: no approximation 1: 3rd-order Taylor approximation(default)
flag.algorithm2 = 1; %outer loop for sigma and theta 1: fminsearchbnd (default) 2: lsqnonlin 3: global with lsqnonlin 4: fsolve
flag.sim = 0; % 1 if using artificial trade share data generating process
flag.sigma = 0; % 1 if sigma is fixed 0 otherwise
flag.second = 85; % second moment is XX% price difference
flag.commoncutoff = 40; % cutoff for the number of destination for price sample. In EIU, every good is available in at least 40 markets among 66.

% simulation parameter
N.draws = 3000000; % number of random seeds

N.sub = 1; % number of different artificial data generating processes in each loop(1 for the first step)
N.sample = 100; % number of sampling price from each simulation, should be 1 for main_firmmmt
N.wruns = 2; % number of iterations for optimal weight matrix
N.runs = 100; % number of different data generating processes in parallel loop for bootstrapping (for standard errors)

% N.sims1 = 1; %20% number of different random number generations for price simulation in parallel loop for bootstrapping (for standard errors)
% N.sims2 = 1;%100; % number of different data generating processes in parallel loop for bootstrapping (for standard errors)
% N.sims = 1;

% initial values
if N.mmts == 3
    sigma = 3.98;
    theta = 3.08;
end
if N.mmts == 1
    sigma = 2; % fixed
    theta = 2.74;
end
 
% shut down unnecessary warning during the gravity regression
warning('off','MATLAB:nearlySingularMatrix')
warning('off','stats:regress:RankDefDesignMat')
warning('off','stats:regress:NoConst')

%%% part adjusting coresize and Matlab version for parfor toolbox
% if parfor toolbox is unavailable ignore this section and replace parfor with for

% Check your parallel computing system (number of cores)
version -date;
datev = datevec(ans);
Matver = datev(1); % Matlab version Matlab 2015a or after uses different parallel computing toolbox
clear v d datev
feature('numcores');
core = ans;

if core > 0
    if isempty(gcp('nocreate'))
        myPool = parpool(core);
    end
end
clear Matver core
%%%
 
% load data
format long

load data_pricemmt % tradeshare, laborforce(L), gross output(Y), EIU price relative to Chile's

N.country = length(Y);
 
% relative tradeshare to domestic
tradeshare = tradeshare./(ones(N.country,1)*diag(tradeshare)');
 
% compute new wages based on wage method and tech (Stilde)
% y and tech do not depend on sigma or theta
ydata = y;
[y,tech,~,taustheta,pred_trdsh,err_var] = est(theta,L,ydata,tradeshareres);

%% price data
% EIU tradable price data for 2004 is used to compute moments from data, mmt_d
% If EIU data is unavailable, skip this section

% relative prices to the reference country's
rel_price = exp_prices./(ones(N.country,1)*exp_prices(N.country,:));

% Note that Chile has 3 missing price observations while the US doesn't
% check if any good is not available in the reference country
zeropriceind = find(exp_prices(N.country,:)==0);

rel_price(:,zeropriceind) = [];

% number of goods in the sample
N.good = length(rel_price);

mmt_d = gen_mmts_pricemmt2(rel_price,y,tradeshare,flag.second,N.mmts);

if N.mmts == 3
    mmt_d = [1.599013256105098
             0.620002648082569
             0.124645869469277];
elseif N.mmts == 1
    mmt_d = [1.599013256105098];
end
%%% I. Estimation for parameters
if N.mmts == 1
    flag.sigma = 1;
end

%%%
% initial values and bounds for solution search
if flag.sigma == 1
    initial = [theta];
elseif flag.sigma == 0
    initial = [sigma;theta];
end

xlwbd = ones(size(initial));
xupbd = 20.*ones(size(initial));

old_sigma = sigma;
old_theta = theta;

stream = RandStream.create('mt19937ar','Seed',20160422);
reset(stream);
RandStream.setGlobalStream(stream);
rnd = rand(N.draws,1);
boot = 122480;

% first step of estimation uses W = I even for over-identification model
flag.weight = 0; % 0 if use W = I; 1 if using optimal weighting matrix
flag.algorithm2 = 1; % use fminsearchbnd
flag.saveW = 0; % W is not stored

options = optimset('Algorithm','interior-point','TolFun',1e-6,'TolX',1e-4,'MaxFunEvals',100,'UseParallel','Always','Display','iter','MaxIter',50);%optimset('TolFun',10^-5,'TolX',10^-3,'Display','iter');
[X,fvals,exitflag,output] = fminsearchbnd(@(x) global_zero_pricemmt(x,L,y,tech,taustheta,pred_trdsh,err_var,mmt_d,rnd,N,flag,boot),initial,xlwbd,xupbd,options);

if flag.sigma == 0
    sigma = X(1);
    theta = X(2);
elseif flag.sigma == 1
    sigma = 2;
    theta = X;
end

% second step of estimation computes optimal weighting matrix following Gourieroux and Monfort(1996)
% this step is only for over-identification model
if N.mmts == 3;
    flag.weight = 1; % 0 if use W = I; 1 if using optimal weighting matrix(for 8 moments)
    flag.algorithm2 = 2; % use lsqnonlin
    
    for i = 1:N.wruns
        
    if i == N.wruns
        flag.saveW = 1; % optimal W is stored to be used for simulations
    end
    
    initial2 = [sigma,theta];
    
    old_sigma = sigma;
    old_theta = theta;

    stream = RandStream.create('mt19937ar','Seed',20160422);
    reset(stream);
    RandStream.setGlobalStream(stream);
    rnd = rand(N.draws,1);
    boot = 122480;


    [X,fvals,diffX,exitflag] = lsqnonlin(@(x) global_zero_pricemmt(x,L,y,tech,taustheta,pred_trdsh,err_var,mmt_d,rnd,N,flag,boot),initial,xlwbd,xupbd,...
    optimset('Algorithm','trust-region-reflective','TolFun',1e-6,'TolX',1e-4,'Display','iter','MaxIter',30,'MaxFunEvals',100,'UseParallel','always'));
    
    sigma = X(1);
    theta = X(2);
    
    end
end

% standard error
flag.saveW = 0;
flag.weight = 1; % load stored optimal weighting matrix
flag.sim = 1;
N.sub = 30; % number of different artificial trade data generating processes in each loop
    
paramat = zeros(2,N.runs);

for k = 1:N.runs


    initial = X;    

    flag.algorithm2 = 2;
    if N.mmts == 3
        flag.weight = 1;

        for i = 1:N.wruns
            % generate different random number for each run k
            stream = RandStream.create('mt19937ar','Seed',20160422+k);
            reset(stream);
            RandStream.setGlobalStream(stream);
            rnd = rand(N.draws,1);            
            
            [Xnew,fvals,diffX,exitflag] = lsqnonlin(@(x) global_zero_pricemmt(x,L,y,tech,taustheta,pred_trdsh,err_var,mmt_d,rnd,N,flag,boot+k),initial,xlwbd,xupbd,...
            optimset('Algorithm','trust-region-reflective','TolFun',1e-6,'TolX',1e-4,'Display','iter','MaxIter',15,'MaxFunEvals',100,'UseParallel','always'));
            initial = Xnew;
        end
        
    elseif N.mmts == 1

        flag.weight = 0;
        % generate different random number for each run k
        stream = RandStream.create('mt19937ar','Seed',20160422+k);
        reset(stream);
        RandStream.setGlobalStream(stream);
        rnd = rand(N.draws,1);    

        [Xnew,fvals,diffX,exitflag] = lsqnonlin(@(x) global_zero_pricemmt(x,L,y,tech,taustheta,pred_trdsh,err_var,mmt_d,rnd,N,flag,boot+k),initial,xlwbd,xupbd,...
        optimset('Algorithm','trust-region-reflective','TolFun',1e-6,'TolX',1e-4,'Display','iter','MaxIter',15,'MaxFunEvals',100,'UseParallel','always'));

    end
   
    paramat(:,k) = Xnew;
                    
end

% standard error
para = mean(paramat,2);
parastd = std(paramat,0,2);

if N.mmts == 3
    str1 = (['result_pricemmt_over.mat']);
elseif N.mmts == 1
    str1 = (['result_pricemmt_exact.mat']);
end
save(str1,'para','parastd')



stream = RandStream.create('mt19937ar','Seed',20160422);
reset(stream);
RandStream.setGlobalStream(stream);
rnd = rand(N.draws,1);  
Jstat = global_zero_pricemmt(para,L,y,tech,taustheta,pred_trdsh,err_var,mmt_d,rnd,N,flag,boot);

%%% II. Simulation for tables and figure

% Table 1 
if N.mmts == 1
    fprintf('Price Moments Case with Exact-Identification Model\n')
elseif N.mmts == 3
    fprintf('Price Moments Case with Over-Identification Model\n')
end
fprintf('estimated sigma is %2.2f(%.3f) and theta is %2.2f(%.3f)\n',para(1),parastd(1),para(2),parastd(2));
if N.mmts == 3
    fprintf('J-stats is %2.2f\n',Jstat)
end

[table1] = gen_mmts_pricemmt(para(1),para(2),y,tech,taustheta,tradeshare,N,1,flag);

if N.mmts == 1
    fprintf('Max Price Gap: %2.2f\n',table1(1));
elseif N.mmts == 3
    fprintf('Max Price Gap: %2.2f\n',table1(1));
    fprintf('85th Price Gap: %2.2f\n',table1(2));    
    fprintf('Cov(gap,wages): %2.2f\n',table1(3));        
end


% Table 2

[~,table2,table3] = gen_tables_firmmmt(para(1),para(2),L,y,tech,taustheta,tradeshare,N,1);

fprintf('Markup Elasticity: %1.2f\n',table2(1));
fprintf('Avg Price-Income Elasticity: %1.2f\n',table2(2));
fprintf('percentage of Exporters in Chile: %2.1f\n',table2(3));
fprintf('percentage of Exporters in the US: %2.1f\n',table2(4));
fprintf('percentage of Exporters, average: %2.1f\n',table2(5));
fprintf('Average Markup: %1.2f\n',table2(6));
fprintf('SD of Log Sales: %1.2f\n',table2(7));

% Table 3
fprintf('Export Intensity: 0-10 percents: %2.1f\n',table3(1));
fprintf('Export Intensity: 10-20 percents: %2.1f\n',table3(2));
fprintf('Export Intensity: 20-30 percents: %2.1f\n',table3(3));
fprintf('Export Intensity: 30-40 percents: %2.1f\n',table3(4));
fprintf('Export Intensity: 40-50 percents: %2.1f\n',table3(5));
fprintf('Export Intensity: 50-60 percents: %2.1f\n',table3(6));
fprintf('Export Intensity: 60-100 percents: %2.1f\n',table3(7));

